@@ -32,6 +32,8 @@ import traceback
 import pyatspi
 import sys
 
+import orca_state
+
 # Used to turn off all debugging.
 #
 LEVEL_OFF = 10000
@@ -113,6 +115,51 @@ debugFile = None
 eventDebugLevel  = LEVEL_FINEST
 eventDebugFilter = None
 
+# What module(s) should be traced if traceit is being used. By default
+# we'll just attend to ourself. (And by default, we will not enable
+# traceit.) Note that enabling this functionality will drag your system
+# to a complete and utter halt and should only be used in extreme
+# desperation by developers who are attempting to reproduce a very
+# specific, immediate issue. Trust me. :-) Disabling braille monitor in
+# this case is also strongly advised.
+#
+TRACE_MODULES = ['orca']
+
+# Specific modules to ignore with traceit.
+#
+TRACE_IGNORE_MODULES = ['traceback', 'linecache', 'locale', 'gettext',
+                        'logging', 'UserDict', 'encodings', 'posixpath',
+                        'genericpath', 're']
+
+# Specific apps to trace with traceit.
+#
+TRACE_APPS = []
+
+# What AT-SPI event(s) should be traced if traceit is being used. By
+# default, we'll trace everything. Examples of what you might wish to
+# do to narrow things down include:
+#
+# TRACE_EVENTS = ['object:state-changed', 'focus:']
+#     (for any and all object:state-changed events plus focus: events)
+# TRACE_EVENTS = ['object:state-changed:selected']
+#     (if you know the exact event type of interest)
+#
+TRACE_EVENTS = []
+
+# What pyatspi role(s) should be traced if traceit is being used. By
+# default, we'll trace everything. An example of what you might wish
+# to do to narrow things down, if you know buttons trigger the problem:
+#
+# TRACE_ROLES = [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_TOGGLE_BUTTON]
+#
+TRACE_ROLES = []
+
+# Whether or not traceit should only trace the work being done when
+# processing an actual event. This is when most bad things happen.
+# So we'll default to True.
+#
+TRACE_ONLY_PROCESSING_EVENTS = True
+
 def printException(level):
     """Prints out information regarding the current exception.
 
@@ -266,13 +313,52 @@ def getAccessibleDetails(level, acc, indent="", includeApp=True):
     return string
 
 
-# The following code has been borrowed from the following URL:
+# The following code originated from the following URL:
 # 
 # http://www.dalkescientific.com/writings/diary/archive/ \
 #                                     2005/04/20/tracing_python_code.html
 #
+import inspect
 import linecache
 
+def _getFileAndModule(frame):
+    filename, module = None, None
+    try:
+        filename = frame.f_globals["__file__"]
+        module = frame.f_globals["__name__"]
+    except:
+        pass
+    else:
+        if (filename.endswith(".pyc") or filename.endswith(".pyo")):
+            filename = filename[:-1]
+
+    return filename, module
+
+def _shouldTraceIt():
+    objEvent = orca_state.currentObjectEvent
+    if not objEvent:
+        return not TRACE_ONLY_PROCESSING_EVENTS
+
+    eventSource = objEvent.source
+    if TRACE_APPS:
+        app = objEvent.host_application or eventSource.getApplication()
+        try:
+            app = objEvent.host_application or eventSource.getApplication()
+        except:
+            pass
+        else:
+            if not app.name in TRACE_APPS:
+                return False
+
+    if TRACE_ROLES and not eventSource.getRole() in TRACE_ROLES:
+        return False
+
+    if TRACE_EVENTS and \
+       not filter(lambda x: x, map(objEvent.type.startswith, TRACE_EVENTS)):
+        return False
+
+    return True
+
 def traceit(frame, event, arg):
     """Line tracing utility to output all lines as they are executed by
     the interpreter.  This is to be used by sys.settrace and is for 
@@ -283,20 +369,44 @@ def traceit(frame, event, arg):
     - event: 'call', 'line', 'return', 'exception', 'c_call', 'c_return',
              or 'c_exception'
     - arg:   depends on the event type (see docs for sys.settrace)
-    """ 
+    """
 
-    if event == "line":
-        lineno = frame.f_lineno
-        filename = frame.f_globals["__file__"]
-        if (filename.endswith(".pyc") or
-            filename.endswith(".pyo")):
-            filename = filename[:-1]
-        name = frame.f_globals["__name__"]
-        if name == "gettext" \
-           or name == "locale" \
-           or name == "posixpath" \
-           or name == "UserDict":
+    if not _shouldTraceIt():
+        return None
+
+    filename, module = _getFileAndModule(frame)
+    if not (filename and module):
+        return traceit
+    if module in TRACE_IGNORE_MODULES:
+        return traceit
+    if TRACE_MODULES and not module.split('.')[0] in TRACE_MODULES:
+        return traceit
+    if not event in ['call', 'line', 'return']:
+        return traceit
+
+    lineno = frame.f_lineno
+    line = linecache.getline(filename, lineno).rstrip()
+    output = 'TRACE %s:%s: %s' % (module, lineno, line)
+
+    if event == 'call':
+        argvals = inspect.getargvalues(frame)
+        keys = filter(lambda x: x != 'self', argvals[0])
+        try:
+            values = map(argvals[3].get, keys)
+        except TypeError:
+            if len(keys) == 1 and isinstance(keys[0], list):
+                values = map(argvals[3].get, keys[0])
+            else:
+                return traceit
+        for i, key in enumerate(keys):
+            output += '\n  ARG %s=%s' % (key, values[i])
+
+    lineElements = line.strip().split()
+    if lineElements and lineElements[0] == 'return':
+        if event == 'line':
             return traceit
-        line = linecache.getline(filename, lineno)
-        println(LEVEL_ALL, "TRACE %s:%s: %s" % (name, lineno, line.rstrip()))
+        output = '%s (rv: %s)' % (output, arg)
+
+    println(LEVEL_ALL, output)
+
     return traceit
